<?php
namespace App\PayModule\Stripe;

use App\PayModule\PmInterface;
use Stripe\Stripe;
use Stripe\Checkout\Session;

class StripeModule implements PmInterface
{
    const SLUG = 'stripe';
    const SUPPORT_CURRENCY = ['USD', 'EUR', 'GBP'];
    
    private $config;

    public function __construct()
    {
        $this->config = PaymentMethod::get_data(self::SLUG);
        if ($this->config) {
            Stripe::setApiKey($this->config->secret_key);
        }
    }

    public function routes()
    {
        Route::prefix('stripe')->group(function () {
            Route::post('/create-checkout', 'StripeController@createCheckout');
            Route::get('/success', 'StripeController@success');
            Route::get('/cancel', 'StripeController@cancel');
            Route::post('/webhook', 'StripeController@webhook');
        });
    }

    public function show_action()
    {
        return [
            'title' => 'Pay with Credit Card',
            'btn_text' => 'Pay Now',
            'btn_class' => 'btn-primary',
            'modal' => false,
            'route' => 'stripe.checkout'
        ];
    }

    public function create_transaction(Request $request)
    {
        try {
            // Create Stripe Checkout Session
            $session = Session::create([
                'payment_method_types' => ['card'],
                'line_items' => [[
                    'price_data' => [
                        'currency' => strtolower($request->currency),
                        'product_data' => [
                            'name' => 'Token Purchase',
                        ],
                        'unit_amount' => $request->amount * 100, // Convert to cents
                    ],
                    'quantity' => 1,
                ]],
                'mode' => 'payment',
                'success_url' => route('stripe.success') . '?session_id={CHECKOUT_SESSION_ID}',
                'cancel_url' => route('stripe.cancel'),
                'metadata' => [
                    'order_id' => $request->order_id
                ]
            ]);

            // Save transaction to database
            $transaction = new Transaction();
            $transaction->user_id = auth()->id();
            $transaction->payment_method = self::SLUG;
            $transaction->payment_id = $session->id;
            $transaction->amount = $request->amount;
            $transaction->currency = $request->currency;
            $transaction->status = 'pending';
            $transaction->save();

            return [
                'success' => true,
                'redirect_url' => $session->url
            ];

        } catch (\Exception $e) {
            return [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }
    }

    public function save_data(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'stripe_public_key' => 'required',
            'stripe_secret_key' => 'required',
            'stripe_webhook_secret' => 'required'
        ]);

        if ($validator->fails()) {
            return false;
        }

        $data = [
            'public_key' => $request->stripe_public_key,
            'secret_key' => $request->stripe_secret_key,
            'webhook_secret' => $request->stripe_webhook_secret,
            'test_mode' => isset($request->test_mode) ? 1 : 0
        ];

        $pm = PaymentMethod::where('payment_method', self::SLUG)->first();
        if (!$pm) {
            $pm = new PaymentMethod();
            $pm->payment_method = self::SLUG;
        }

        $pm->title = $request->title;
        $pm->description = $request->description;
        $pm->status = isset($request->status) ? 'active' : 'inactive';
        $pm->data = json_encode($data);

        return $pm->save();
    }
}